from .csvrequest import Ui_NewPlot
from .curveproperties import Ui_CurveProperties
from .curveselect import Ui_CurveSelection
from .cycledetect import Ui_CycleDetection
from .datetime import Ui_SetDateTime
from .filter import Ui_Filter
from .loopwidget import Ui_LoopWidget
from .mainwindow import Ui_MainWindow
from .periodexport import Ui_PeriodExport
from .poiwidget import Ui_POISelectorWidget
from .setuppuloop import Ui_SetupPULoop
